local _G = getfenv(0)
local VERSION = tonumber(("$Revision: 41994 $"):match("%d+"))
local L = AceLibrary("AceLocale-2.2"):new("AuldLangSyne_Fu")

local tablet = AceLibrary("Tablet-2.0")
local BC = AceLibrary("Babble-Class-2.2")

local AuldLangSyne = AuldLangSyne
local Fu = AuldLangSyne:NewModule("Fu", "AceHook-2.1", "AceEvent-2.0", "FuBarPlugin-2.0")
if AuldLangSyne.revision < VERSION then
	AuldLangSyne.version = "r" .. VERSION
	AuldLangSyne.revision = VERSION
	AuldLangSyne.date = ("$Date: 2007-06-28 01:46:16 -0400 (Thu, 28 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local new, del, newHash, newSet, deepCopy, deepDel = AuldLangSyne.new, AuldLangSyne.del, AuldLangSyne.newHash, AuldLangSyne.newSet, AuldLangSyne.deepCopy, AuldLangSyne.deepDel

Fu.name = "AuldLangSyneFu"
Fu.title = "AuldLangSyneFu"
Fu.clickableTooltip = true
Fu.hasIcon = "Interface\\FriendsFrame\\FriendsFrameScrollIcon"

function Fu:OnInitialize(name)
	self.db = AuldLangSyne:AcquireDBNamespace("Fu")
	--AuldLangSyne:RegisterDefaults("Fu", "profile", {})
	
	--[[self.menu = {
		handler = Fu,
		name = "Fu", type = "group",
		desc = "FuBar plugin options",
		args = {
			
		},
	}--]]
	self.OnMenuRequest = AuldLangSyne.menu
	
	self.friends = new()
	self.friendLevels = new()
	self.friendClasses = new()
	self.friendAreas = new()
	self.friendStatus = new()
	self.friendConnected = new()
	self.numFriends = 0
	self.numFriendsOnline = 0
end

function Fu:OnEnable(first)
	self:RegisterEvent("FRIENDLIST_UPDATE", "Update")
	self:SecureHook("FriendsList_Update", "Update")
end

--[[
function Fu:OnDisable()
	
end
--]]

------------------------------------------------------------------------
-- FuBar methods
------------------------------------------------------------------------

function Fu:OnDataUpdate()
	del(self.friends)
	del(self.friendLevels)
	del(self.friendClasses)
	del(self.friendAreas)
	del(self.friendStatus)
	del(self.friendConnected)
	local friends = new()
	local friendLevels = new()
	local friendClasses = new()
	local friendAreas = new()
	local friendStatus = new()
	local friendConnected = new()
	local numFriends = GetNumFriends()
	local numFriendsOnline = 0
	
	for i=1, numFriends do
		local name, level, class, area, connected, status = GetFriendInfo(i)
		
		if name then
			table.insert(friends, name)
			if connected then
				numFriendsOnline = numFriendsOnline + 1
				friendLevels[name] = level
				friendClasses[name] = class
				friendAreas[name] = area
				friendStatus[name] = status
			end
			friendConnected[name] = connected
		end
	end
	
	self.friends = friends
	self.friendLevels = friendLevels
	self.friendClasses = friendClasses
	self.friendAreas = friendAreas
	self.friendStatus = friendStatus
	self.friendConnected = friendConnected
	self.numFriends = numFriends
	self.numFriendsOnline = numFriendsOnline
end

function Fu:OnTextUpdate()
	self:SetText(self.numFriendsOnline..'/'..self.numFriends)
end

do
	local function showNote(note)
		GameTooltip:SetOwner(_G.this, 'ANCHOR_RIGHT', 7, -18)
		GameTooltip:ClearLines()
		GameTooltip:AddLine(note, 1, 1, 1, true)
		GameTooltip:Show()
	end
	local function editNote(name)
		local notes = AuldLangSyne:GetModule("Note")
		if notes.editFrame:IsVisible() and notes.editFrame.name == name then
			notes.editFrame.name = ""
			notes.editFrame.type = ""
			notes.editFrame:Hide()
		else
			notes.editFrame.name = name
			notes.editFrame.type = 'friend'

			notes.editFrame.editbox:SetText(notes.db.realm.friend[name] or '')
			notes.editFrame.edit_text:SetText(format(AceLibrary("AceLocale-2.2"):new("AuldLangSyne_Note")["Editing note for %s"], name))
			notes.editFrame:Show()
		end
	end
	function Fu:OnTooltipUpdate()
		local notes = AuldLangSyne:HasModule("Note") and AuldLangSyne:GetModule("Note")
		local info = AuldLangSyne:HasModule("Info") and AuldLangSyne:GetModule("Info")
		local cat = tablet:AddCategory(
			'id', 'friends', 'columns', 6,
			'text', 'Name', 'text2', 'Level', 'text3', 'Class', 'text4', 'Area', 'text5', 'Status/Seen'
		)
		for _,name in pairs(self.friends) do
			local r,g,b, class, area, status, note
			if self.friendConnected[name] then
				r,g,b = 1,1,1
				level = self.friendLevels[name]
				class = self.friendClasses[name]
				area = self.friendAreas[name]
				status = self.friendStatus[name]
			else
				local thisinfo = info and info.db.realm[name]
				r,g,b = 0.5,0.5,0.5
				level = thisinfo and thisinfo.level or '?'
				class = thisinfo and thisinfo.class or '?'
				area = ''
				status = thisinfo and info:HowLongAgo(thisinfo.updated) or '?'
			end
			class = "|cff"..BC:GetHexColor(class)..class.."|r"
			note = notes and notes.db.realm.friend[name]
			cat:AddLine(
				'text', name, 'textR', r, 'textG', g, 'textB', b,
				'text2', level,
				'text3', class,
				'text4', area,
				'text5', status,
				'func', notes and editNote, 'arg1', name,
				'onEnterFunc', note and showNote, 'onEnterArg1', note,
				'onLeaveFunc', note and GameTooltip.Hide or nil, 'onLeaveArg1', note and GameTooltip
			)
		end
	end
end
